package main

import (
	"fmt"
	"os"
	"os/exec"
	"path"
	"strings"

	"codeberg.org/emersion/go-scfg"
)

type account struct {
	endpoint    string
	username    string
	token       string
	tokenCmd    string
	password    string
	passwordCmd string
}

func loadConfig() (*account, error) {
	home, err := os.UserConfigDir()
	if err != nil {
		return nil, fmt.Errorf("cannot determine config home: %w", err)
	}

	path := path.Join(home, "mjmap", "config.scfg")
	file, err := os.Open(path)
	if err != nil {
		return nil, err
	}

	root, err := scfg.Read(file)
	if err != nil {
		return nil, err
	}

	cfg := root.Get("account")
	if cfg == nil {
		return nil, fmt.Errorf("no account found")
	}

	acct := &account{}
	for _, key := range cfg.Children {
		switch key.Name {
		case "token":
			if err := key.ParseParams(&acct.token); err != nil {
				return nil, err
			}
		case "session-url":
			if err := key.ParseParams(&acct.endpoint); err != nil {
				return nil, err
			}
		case "username":
			if err := key.ParseParams(&acct.username); err != nil {
				return nil, err
			}
		case "token-cmd":
			if err := key.ParseParams(&acct.tokenCmd); err != nil {
				return nil, err
			}
		case "password":
			if err := key.ParseParams(&acct.password); err != nil {
				return nil, err
			}
		case "password-cmd":
			if err := key.ParseParams(&acct.passwordCmd); err != nil {
				return nil, err
			}
		}
	}

	if acct.tokenCmd != "" && acct.token == "" {
		cmd := exec.Command("sh", "-c", acct.tokenCmd)
		out, err := cmd.Output()
		if err != nil {
			return nil, err
		}
		lines := strings.Split(string(out), "\n")
		if len(lines) == 0 {
			return nil, fmt.Errorf("No output from token-cmd")
		}
		acct.token = lines[0]
	} else if acct.passwordCmd != "" && acct.password == "" {
		cmd := exec.Command("sh", "-c", acct.passwordCmd)
		out, err := cmd.Output()
		if err != nil {
			return nil, err
		}
		lines := strings.Split(string(out), "\n")
		if len(lines) == 0 {
			return nil, fmt.Errorf("No output from password-cmd")
		}
		acct.password = lines[0]
	}

	return acct, nil
}
